@@ -2,7 +2,7 @@
 *     xfpt - Simple ASCII->Docbook processor     *
 *************************************************/
 
-/* Copyright (c) University of Cambridge, 2008 */
+/* Copyright (c) University of Cambridge, 2018 */
 /* Written by Philip Hazel. */
 
 /* This module contains code for processing a line that starts with a dot. */
@@ -80,7 +80,7 @@ while (!done)
     {
     macroexe *temp = macrocurrent;
     macrocurrent = macrocurrent->prev;
-    free(temp);
+    macro_free(temp);
     from_type_ptr--;
     break;
     }
@@ -282,8 +282,6 @@ uschar *pp, *q;
 int length, term;
 flagstr *f, **ff;
 
-f = misc_malloc(sizeof(flagstr));
-
 /* Check that the flag starts with & and then get a copy of the rest of it. */
 
 if (*p++ != '&') { error(9); return; }
@@ -292,14 +290,17 @@ for (pp = p; *pp != 0 && !isspace(*pp); pp++);
 length = pp - p;
 if (length == 0) { error(10); return; }
 
+f = misc_malloc(sizeof(flagstr));
 f->length1 = length;
+f->length2 = 0;   /* In case of error call to free */
 f->flag1 = misc_copystring(p, length);
+f->rep1 = NULL;   /* In case of error call to free */
 
 /* Now look backwards from the end of the line and find the last quoted string
 that is there. */
 
 q = pp + Ustrlen(pp);
-if (*(--q) != '\"' && *q != '\'') { error(11, thisdir); return; }
+if (*(--q) != '\"' && *q != '\'') { error(11, thisdir); flag_free(f); return; }
 
 term = *q;
 while (--q > pp)
@@ -307,7 +308,7 @@ while (--q > pp)
   if (*q == term) { if (q[-1] == term) q--; else break; }
   }
 
-if (q <= pp) { error(11, thisdir); return; }
+if (q <= pp) { error(11, thisdir); flag_free(f); return; }
 
 /* If there's nothing between pp and q, we have the definition of a single,
 non-paired flag. */
@@ -316,7 +317,6 @@ while (isspace(*pp)) pp++;
 if (pp == q)
   {
   f->rep1 = misc_readstring(q, NULL, NULL, 0);
-  f->length2 = 0;
   f->flag2 = f->rep2 = NULL;
   }
 
@@ -328,13 +328,13 @@ else
   p = pp;
   while (*pp != 0 && !isspace(*pp)) pp++;
   length = pp - p;
-  if (length == 0) { error(10); return; }
+  if (length == 0) { error(10); flag_free(f); return; }
   f->length2 = length;
   f->flag2 = misc_copystring(p, length);
   while (isspace(*pp)) pp++;
-  if (*pp != '\"' && *pp != '\'') { error(11, thisdir); return; }
+  if (*pp != '\"' && *pp != '\'') { error(11, thisdir); flag_free(f); return; }
   f->rep1 = misc_readstring(pp, &length, NULL, 0);
-  if (pp + length >= q) { error(11, thisdir); return; }
+  if (pp + length >= q) { error(11, thisdir); flag_free(f); return; }
   }
 
 /* Successfully defined the flag(s). Attach the block to the chain. The order
@@ -554,9 +554,10 @@ static void
 do_nonl(uschar *p)
 {
 static uschar nonlbuffer[INBUFFSIZE];
-int len = Ustrlen(p) + 1;
-if (len > INBUFFSIZE) len = INBUFFSIZE;
-Ustrncpy(nonlbuffer, p, len);
+int len = Ustrlen(p);
+if (len > INBUFFSIZE - 1) len = INBUFFSIZE - 1;
+(void)memcpy(nonlbuffer, p, len);
+nonlbuffer[len] = 0;
 next_line = nonlbuffer;
 }
 
