@@ -2,14 +2,16 @@
 *     xfpt - Simple ASCII->Docbook processor     *
 *************************************************/
 
-/* Copyright (c) University of Cambridge, 2007 */
+/* Copyright (c) University of Cambridge, 2018 */
 /* Written by Philip Hazel. */
 
 /* This header defines all the global functions. */
 
 extern void        dot_process(uschar *);
 extern void        error(int, ...);
+extern void        flag_free(flagstr *);
 extern void        literal_process(uschar *);
+extern void        macro_free(macroexe *);
 extern uschar     *misc_copystring(uschar *, int);
 extern void        misc_detrail(uschar *);
 extern void       *misc_malloc(int);
